#ifndef _SHADERPROGRAM_H__
#define _SHADERPROGRAM_H__

#include "common.h"
#include "glInterface.h"

#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <string>
#include <map>


namespace CHCDemoEngine 
{ 

class FrameBufferObject;
class Vector3;
class Camera;
class Matrix4x4;
class ShadowMap;
class DirectionalLight;

typedef std::map<std::string, CGparameter> CGParameterMap;
typedef std::vector<CGparameter> CGParameterArray;

struct ParameterPair1f
{
	unsigned int mParameterIdx;
	float mValue;
};

class GPUProgramParameters
{
	struct ParameterValue1f
	{
		unsigned int mParameterIdx;
		float mValue;
	};

	struct ParameterValue2f
	{
		unsigned int mParameterIdx;
		float mValue;
	};

	struct ParameterValue3f
	{
		unsigned int mParameterIdx;
		float mValue;
	};
};



class ShaderProgram
{
public:

	ShaderProgram(CGprogram program): mProgram(program) {}

	ShaderProgram(CGcontext context, 
		          const std::string &filename, 
				  CGprofile profile,
				  const std::string &functionName);

	~ShaderProgram() { if (mProgram != NULL) cgDestroyProgram(mProgram); }

	void SetValue1f(const std::string &name, float value);
	void SetValue2f(const std::string &name, float val1, float val2);
	void SetValue3f(const std::string &name, float val1, float val, float val3);

	void SetArray1f(const std::string &name, float *vals, int numElements);
	void SetArray2f(const std::string &name, float *vals, int numElements);
	void SetArray3f(const std::string &name, float *vals, int numElements);
	/** Sets the matrix parameter.
	*/
	void SetMatrix(const std::string &name, const Matrix4x4 &mat);
	/** Sets the texture parameter.
	*/
	void SetTexture(const std::string &name, unsigned int tex);
	/** Enable / disable a texture parameter.
	*/
	void EnableTexture(const std::string &name);

	void DisableTexture(const std::string &name);
	/** Binds the program.
	*/
	void Bind();
	/** Returns true if this program is valid.
	*/
	inline bool IsValid() { return mProgram != NULL; }


protected:

	CGparameter GetOrCreateParameter(const std::string &name);

	CGParameterMap mParamHash;
	//CGParameterArray mParameters;

	CGParameterArray mTextureParams;

	CGprogram mProgram;
};


typedef  std::vector<ShaderProgram *> ShaderContainer;

} // namespace 

#endif // _ShaderProgram_H__